package org.gcube.application.geoportal.clients.legacy;

import org.gcube.application.cms.tests.model.concessioni.TestConcessioniFilters;
import org.gcube.application.cms.tests.model.concessioni.TestConcessioniModel;
import org.gcube.application.cms.tests.model.concessioni.TestConcessioniQueries;
import org.gcube.application.geoportal.client.utils.Queries;
import org.gcube.application.geoportal.client.utils.Serialization;
import org.gcube.application.geoportal.clients.BasicVreTests;
import org.gcube.application.geoportal.common.model.legacy.*;
import org.gcube.application.geoportal.common.model.legacy.Concessione.Paths;
import org.gcube.application.geoportal.common.model.legacy.report.ValidationReport.ValidationStatus;
import org.gcube.application.geoportal.common.model.rest.AddSectionToConcessioneRequest;
import org.gcube.application.geoportal.common.rest.MongoConcessioni;
import org.gcube.application.geoportal.common.utils.FileSets;
import org.gcube.application.geoportal.common.utils.StorageUtils;
import org.gcube.application.geoportal.common.utils.tests.GCubeTest;
import org.junit.Test;

import java.io.File;
import java.util.Iterator;
import java.util.concurrent.atomic.AtomicLong;

import static org.gcube.application.geoportal.client.plugins.GeoportalAbstractPlugin.mongoConcessioni;
import static org.junit.Assert.*;
import static org.junit.Assume.assumeTrue;

public class StatelessClientTests extends BasicVreTests {

	
	private final MongoConcessioni client=mongoConcessioni().build();

	@Test
	public void getConfiguration() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		System.out.println(client.getCurrentConfiguration());
	}

	@Test
	public void searches() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		TestConcessioniFilters.filters.forEach((k, v)->{
			try {
				System.out.println("Count for " + k + "\t" + count(client.search(v)));
			}catch (Exception e){
				throw new RuntimeException(e);
			}
		});


	}


	@Test
	public void query() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		System.out.println("FRA "+ count(client.query(
				Queries.parse(TestConcessioniQueries.queries.get("emptyresult.json")))));

		// No Transformation
		System.out.print("First Registered \t");
		Iterator<Concessione> queriedDocuments=client.query(
				Queries.parse(TestConcessioniQueries.queries.get("firstRegistered.json")));
		// Expected one result
		assertTrue(count(queriedDocuments)==1);



		System.out.print("Last Registered \t");
		// Expected one result
		queriedDocuments=client.query(
				Queries.parse(TestConcessioniQueries.queries.get("lastRegistered.json")));
		assertTrue(count(queriedDocuments)==1);

		queriedDocuments.forEachRemaining((Concessione c)->{System.out.println(c.getNome());});


		// Transformations
		System.out.println(
				client.queryForJSON(
						Queries.parse(TestConcessioniQueries.queries.get("lastNameRegisteredByFabio.json"))));

		System.out.println(
				client.queryForJSON(
						Queries.parse(TestConcessioniQueries.queries.get("publicationWarningMessages.json"))));

//			String query=Files.readFileAsString(filterFile.getAbsolutePath(), Charset.defaultCharset());
//			System.out.println("Count for "+filterFile.getName()+"\t"+ count(client.search(query)));
//		}
	}

	@Test
	public void testList() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		final AtomicLong counter=new AtomicLong();
		long before=System.currentTimeMillis();
		client.getList().forEachRemaining((Concessione c)-> {counter.addAndGet(1);});
		System.out.println("Loaded "+counter+" in "+(System.currentTimeMillis()-before)+" ms");
	}

	@Test
	public void testCreateNew() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		Concessione c= client.createNew(TestConcessioniModel.prepareEmptyConcessione());
		assertNotNull(c);
		assertNotNull(c.getMongo_id());
	}

	@Test
	public void testReplace() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		Concessione testObject= client.createNew(TestConcessioniModel.prepareEmptyConcessione());
		String title="My new shiny Title";
		testObject.setNome(title);
		Concessione c1=client.replace(testObject);
		assertEquals(title, c1.getNome());
	}

	@Test
	public void testUploadFileSet() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		Concessione testObject= client.createNew(TestConcessioniModel.prepareConcessione());
		AddSectionToConcessioneRequest request=
				Serialization.convert(
//				FileSets.prepareRequest(new StorageUtils(),Paths.RELAZIONE,new File(TestConcessioniModel.getBaseFolder(),"relazione.pdf"));
		FileSets.build(Paths.ABSTRACT_RELAZIONE,null,null).add(
				new StorageUtils().putOntoStorage(new File(TestConcessioniModel.getBaseFolder(),"relazione.pdf"),"San Mauro_drone totale.JPG"))
				.getTheRequest(),AddSectionToConcessioneRequest.class);

		testObject= client.registerFileSet(testObject.getMongo_id(), request);

		assertNotNull(testObject.getContentByPath(Paths.ABSTRACT_RELAZIONE).getActualContent());
		assertNotNull(testObject.getContentByPath(Paths.ABSTRACT_RELAZIONE).getActualContent().get(0));
	}



	@Test
	public void testDeleteById() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		Concessione c= client.createNew(TestConcessioniModel.prepareEmptyConcessione());
		client.deleteById(c.getMongo_id());
	}


	@Test
	public void testPublish() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		Concessione c=prepare();

		assertTrue(c.getReport().getStatus().equals(ValidationStatus.PASSED));


	}

	@Test
	public void testCleanFileSet() throws Exception {
		assumeTrue(GCubeTest.isTestInfrastructureEnabled());
		Concessione testObject=prepare();
		System.out.println("Object is "+testObject.getPosizionamentoScavo());
		//Precheck to be sure
		assertFalse(testObject.getPosizionamentoScavo().getActualContent().isEmpty());
		assertFalse(testObject.getPianteFineScavo().get(0).getActualContent().isEmpty());

		client.unPublish(testObject.getMongo_id());

		// check unpublish
		testObject= client.getById(testObject.getMongo_id());
		for(LayerConcessione l:testObject.getPianteFineScavo())
			for(PersistedContent pc: l.getActualContent())
				if(pc instanceof GeoServerContent) throw new Exception ("Concessione not properly unpublished");

		for(PersistedContent pc: testObject.getPosizionamentoScavo().getActualContent())
			if(pc instanceof GeoServerContent) throw new Exception ("Concessione not properly unpublished");


		//Clear pos
		testObject=client.cleanFileSet(testObject.getMongo_id(),Paths.POSIZIONAMENTO);
		assertTrue(testObject.getPosizionamentoScavo().getActualContent().isEmpty());

		//Clear pianta [0]
		testObject =client.cleanFileSet(testObject.getMongo_id(),Paths.piantaByIndex(0));
		assertTrue(testObject.getPianteFineScavo().get(0).getActualContent().isEmpty());
	}


	private Concessione prepare() throws Exception {
		int numImgs=1;
		Concessione c= client.createNew(TestConcessioniModel.prepareConcessione(1,numImgs));
		StorageUtils storage = new StorageUtils();

		c.getRelazioneScavo().setPolicy(AccessPolicy.EMBARGOED);
		String mongoId=c.getMongo_id();
		c=client.update(mongoId, Serialization.write(c));

		assertEquals(AccessPolicy.EMBARGOED,c.getRelazioneScavo().getPolicy());

		c=client.registerFileSet(mongoId,
				Serialization.convert(FileSets.prepareRequest(storage,Paths.RELAZIONE,null,null,
						new File(TestConcessioniModel.getBaseFolder(),"relazione.pdf")), AddSectionToConcessioneRequest.class));

		assertEquals(AccessPolicy.EMBARGOED,c.getRelazioneScavo().getPolicy());

		c=client.registerFileSet(mongoId,
				Serialization.convert(
						FileSets.prepareRequest(storage,Paths.ABSTRACT_RELAZIONE,null, null, new File(TestConcessioniModel.getBaseFolder(),"relazione.pdf")),AddSectionToConcessioneRequest.class));

		assertEquals(AccessPolicy.EMBARGOED,c.getRelazioneScavo().getPolicy());

		for(int i=0;i<numImgs;i++)
			c=client.registerFileSet(mongoId,
					Serialization.convert(
							FileSets.build(Paths.imgByIndex(i),null,null).
							add(storage.putOntoStorage(new File(TestConcessioniModel.getBaseFolder(),"immagine"+(i+1)+".png"),
									i+"San Mauro_drone totale.JPG")).getTheRequest(),AddSectionToConcessioneRequest.class));

		assertEquals(AccessPolicy.EMBARGOED,c.getRelazioneScavo().getPolicy());

		c=client.registerFileSet(mongoId,
				Serialization.convert(
						FileSets.prepareRequest(storage,Paths.POSIZIONAMENTO,null,null,new File(TestConcessioniModel.getBaseFolder(),
								"pos.shp")),AddSectionToConcessioneRequest.class));

		assertEquals(AccessPolicy.EMBARGOED,c.getRelazioneScavo().getPolicy());

		c=client.registerFileSet(mongoId,
				Serialization.convert(
						FileSets.prepareRequest(storage,Paths.piantaByIndex(0),null,null,
								new File(TestConcessioniModel.getBaseFolder(),"pianta.shp")),AddSectionToConcessioneRequest.class));

		assertEquals(AccessPolicy.EMBARGOED,c.getRelazioneScavo().getPolicy());

		c=client.publish(mongoId);
		assertEquals(AccessPolicy.EMBARGOED,c.getRelazioneScavo().getPolicy());

		return c;
	}

	// UTILS

	public static long count(Iterator<?> iterator){
		AtomicLong l=new AtomicLong(0);
		iterator.forEachRemaining(el->{l.incrementAndGet();});
		return l.get();
	}


}
